/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Clippings.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2007-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


var gDlgArgs;
var gDlg;
var gDeck;
var gStartBtn;
var gCancelBtn;
var gStatusMsg;
var gProgressMeter;
var gIsStarted = false;
var gIsFinished = false;
var gAppUtils;
var gStrBundle;


function $(aID)
{
  return document.getElementById(aID);
}


function doAlert(aMessage)
{
  var title = gStrBundle.getString('appName');
  gAppUtils.alertEx(title, aMessage);
}


function initDlg() 
{
  gDlgArgs = window.arguments[0];
  gAppUtils = new aeClippingsUtils();

  gStrBundle = $("ae-clippings-strings");
  gDlg = $("ae-common-clippings");
  gDeck = $("content-deck");
  gCancelBtn = gDlg.getButton("cancel");
  gStartBtn = gDlg.getButton("accept");
  gStatusMsg = $("status-msg");
  gProgressMeter = $("progress-meter");

  var thisHostApp = gAppUtils.getHostAppName();
  var otherHostApp = getOtherHostAppName();
  var introElt = $("migration-intro");
  var remindElt = $("migration-reminder");
  var intro = gStrBundle.getFormattedString("migrationIntro", [thisHostApp]);
  var remind = gStrBundle.getFormattedString("migrationRemind", [otherHostApp]);
  introElt.appendChild(document.createTextNode(intro));
  remindElt.appendChild(document.createTextNode(remind));

  // If common clippings is already enabled, don't allow this wizard
  // to be run again.
  if (gAppUtils.isCommonDataSourceEnabled()) {
    gDeck.selectedIndex = 0;
    gStartBtn.style.visibility = "hidden";
    gDlg.defaultButton = "cancel";
    gCancelBtn.focus();
    gCancelBtn.label = gStrBundle.getString("btnClose");
  }
}


function doCancel() 
{
  // Cancel migration while it is in progress
  if (gIsStarted && !gIsFinished) {
    // User should be able to cancel the migration process unless the
    // common data source file has been fully populated with migrated clippings
    // AND the pref "clippings.datasource.common" has been set to 'true'.
    gAppUtils.log("Migration cancelled by user");

    gDlgArgs.clippingsSvc.reset();
    var profileDirURL = gAppUtils.getCurrentProfileURL();
    try {
      gDlgArgs.clippingsSvc.getDataSource(profileDirURL + gAppUtils.CLIPDAT_FILE_NAME);
      gDlgArgs.clippingsSvc.setBackupDir(profileDirURL + gAppUtils.BACKUP_DIR_NAME);
    }
    catch (e) {
      gAppUtils.log(e);
      doAlert(gStrBundle.getString("errorCancelMigration"));
    }

    gDlgArgs.userCancel = true;
  }
  // Finished migration
  else if (gIsFinished) {
    // Set the migration flag here, because the reload event on the opener
    // (i.e., clipman) fires before returning control to the function that
    // opened this window.
    window.opener.gJustMigrated = true;

    gDlgArgs.clippingsSvc.notifyMigrationDone();
    gDlgArgs.userCancel = false;
  }
  // Cancel migration from 1st screen
  else {
    gDlgArgs.userCancel = true;
  }

  return true;
}


function getOtherHostAppName()
{
  var rv;
  var thisHostAppID = gAppUtils.getHostAppID();

  if (thisHostAppID == gAppUtils.HOSTAPP_FX_GUID) {
    rv = gStrBundle.getString("tb");
  }
  else if (thisHostAppID == gAppUtils.HOSTAPP_TB_GUID) {
    rv = gStrBundle.getString("fx");
  }

  return rv;
}


function start() 
{
  gStartBtn.disabled = true; 
  gDeck.selectedIndex = 2;
  gDlg.defaultButton = "cancel";
  gCancelBtn.focus();
  gStatusMsg.value = gStrBundle.getString("prepMigration");

  if (!gAppUtils.isCommonDataSourceEnabled() && commonDSFileExists()) {
    gAppUtils.log("Common datasource file detected.");
    // Leave the file alone, because it may have been created by the
    // migrator utility in the other host app.
    // Problem - file may already exist because a previous migration attempt
    // from either host app was cancelled by the user.  In this situation,
    // there is no way to reliably determine if the file exists because of the
    // aborted migration, or because it was created from the other host app.
  }

  // Save and back up datasource before migration
  try {
    gDlgArgs.clippingsSvc.flushDataSrcEx(true);
  }
  catch (e) {
    doAlert(gStrBundle.getString("errorSaveFailedCannotMigrate"));
    return doCancel();
  }

  gIsStarted = true;

  window.setTimeout(function () { initMigration(); }, 500);

  return false;
}


function commonDSFileExists()
{
  var homeDir = gAppUtils.getHomeDir();
  var dsFile = homeDir.clone();
  dsFile.append(gAppUtils.CLIPDAT_FILE_NAME);

  return (dsFile.exists() && dsFile.isFile());
}


function getHomeDirURL()
{
  var homeDir = gAppUtils.getHomeDir();
  var fph = Components.classes["@mozilla.org/network/protocol;1?name=file"]
                      .createInstance(Components.interfaces
				                .nsIFileProtocolHandler);
  return fph.getURLSpecFromFile(homeDir);
}


function initMigration()
{
  // Reset nsIClippingsService
  gDlgArgs.clippingsSvc.reset();
  gProgressMeter.value = 15;

  // Make nsIClippingsService point to the new common data source
  var homeDirURL = getHomeDirURL();
  try {
    gDlgArgs.clippingsSvc.getDataSource(homeDirURL + gAppUtils.CLIPDAT_FILE_NAME);
    gDlgArgs.clippingsSvc.setBackupDir(homeDirURL + gAppUtils.BACKUP_DIR_NAME);
  }
  catch (e) {
    doAlert(gStrBundle.getString("errorInitCommonDS"));
    return doCancel;
  }
  gProgressMeter.value = 30;  
  gStatusMsg.value = gStrBundle.getString("migrationInProgress");

  window.setTimeout(function () { doMigration(); }, 1000);
}


function doMigration()
{
  var oldDSFileURL = gAppUtils.getCurrentProfileURL() + gAppUtils.CLIPDAT_FILE_NAME;
  gProgressMeter.value = 50;

  // Point of no return
  gCancelBtn.disabled = true;
 
  try {
    gDlgArgs.clippingsSvc.importFromFile(oldDSFileURL, true, false, {});
  }
  catch (e if e.result == Components.results.NS_ERROR_NOT_INITIALIZED) {
    doAlert(gStrBundle.getString('errorMigrationFailedNoDS'));
    return doCancel();
  }
  catch (e) {
    gAppUtils.log(e);
    doAlert(gStrBundle.getString("errorMigration"));
    return doCancel();
  }
  gProgressMeter.value = 75;

  window.setTimeout(function () { saveCommonDataSource(); }, 800);
}


function saveCommonDataSource()
{
  try {
    gDlgArgs.clippingsSvc.flushDataSrcEx(true);
  }
  catch (e) {
    gAppUtils.log(e);
    doAlert(gStrBundle.getString("alertSaveFailed"));
  }
  gProgressMeter.value = 90;

  window.setTimeout(function () { finalizeMigration(); }, 300);
}


function finalizeMigration()
{
  gStatusMsg.value = gStrBundle.getString("finishMigration");

  gAppUtils.setPref("clippings.datasource.common", true);
  gProgressMeter.value = 100;

  window.setTimeout(function () { finish(); }, 2500);
}


function finish()
{
  gCancelBtn.disabled = false;
  gStartBtn.style.visibility = "hidden";
  gCancelBtn.label = gStrBundle.getString("btnFinish");

  gDeck.selectedIndex = 3;
  gIsFinished = true;
}



